#include <ast.h>
#include <stdlib.h>
#include <util.h>

ast_t *
ast_new ()
{
  ast_t *ast = gtree_new (ast_t);
  return ast;
}

void
__ast_print (gtree_t * root, int depth)
{
  int i;
  char *indentation = "  ";
  for (i = 0; i < depth; i++)
    printf (indentation);
  printf ("%s\n", AST (root)->attrs[1]);
  int j;

  for (i = 1; i < (AST (root)->nattrs / 2); i++)
    {
      for (j = 0; j < (depth + 1); j++)
	printf (indentation);
      printf ("%s = %s\n", AST (root)->attrs[2 * i],
	      AST (root)->attrs[1 + 2 * i]);
    }
}

void
__ast_free (gtree_t * root)
{

  int i;
  for (i = 0; i < AST (root)->nattrs; i++)
    free (AST (root)->attrs[i]);
  free (AST (root)->attrs);

  free (root);
  memset (root, 0, sizeof (gtree_t));
  root = NULL;
}


void
ast_loaddot_nodes (ast_t * ast, Agnode_t * root)
{

  Agedge_t *e;

  assert (root);

  char **argv;
  int argc = get_attr (root, &argv);
  int i;

  for (i = 0; i < argc; i += 2)
    {
      assert (argv[i + 0]);
      assert (argv[i + 1]);
    }

  ast->attrs = argv;
  ast->nattrs = argc;

  for_each_edge (e, root, root->graph)
  {
    Agnode_t *head = e->head;
    ast_t *child = gtree_append (ast, ast_t);
    ast_loaddot_nodes (child, head);
  }
  return;
}


ast_t *
ast_loaddot (ast_t * ast, char *filename)
{

  aginit ();

  FILE *fd = fopen (filename, "r");

  if (!fd)
    {
      panic ("can't open %s ( parsing failed )", filename);
    }

  Agraph_t *graph = agread (fd);

  fclose (fd);

  Agnode_t *root = agfstnode (graph);

  ast_loaddot_nodes (ast, root);

}

void
ast_print (ast_t * ast)
{
  gtree_transverse (GTREE (ast), 0, __ast_print);
}

ast_t *
ast_free (ast_t * ast)
{
  gtree_postorder (GTREE (ast), __ast_free);
  return NULL;
}
