#include <trunit.h>
#include <verilog.h>
#include <vhdl.h>
#include <util.h>
#include <ast.h>

static inline char *
file_name (char *str)
{
  int i;
  for (i = strlen (str); i >= 0; i--)
    if (str[i] == '/')
      return &str[i + 1];
  return str;
}

int
main (int argc, char **argv)
{

  if (argc < 2)
    {
      printf ("usage : gsc module.cpp\n");
      exit (-1);
    }

  //parse options
  opts_t options = parse_opts (argc, argv);
  int result = parse (options.keystone.argc, options.keystone.argv);
  if (!result)
    exit (-1);

  //invoke translator
  char *filename;
  asprintf (&filename, "%s.ast.dot", file_name (argv[1]));

  //loads translation unit
  trunit_t *unit = trunit_new (filename);
  free (filename);

  //if we have an output file, open it ...
  FILE *outfile;
  if (options.outfile != NULL)
    {
      outfile = fopen (options.outfile, "w");
      assert (outfile);
    }
  else
    {
      outfile = stdout;
    }

  //translates to verilog
  trunit_translate (outfile, unit, verilog);

  fclose (outfile);

  //deletes temporary files
  asprintf (&filename, "rm -f %s.ast.dot", file_name (argv[1]));
  assert (system (filename) != -1);
  free (filename);

  //free memory
  trunit_free (unit);

  return 0;
}
