#include <trunit.h>
#include <util.h>
#include <module.h>
#include <method.h>

trunit_t *gunit;

inline void
trunit_translate (FILE * fd, trunit_t * unit,
		  void (*algorithim) (FILE * fd, trunit_t *))
{

  algorithim (fd, unit);

}

inline void
enumeration_load (linkedlist_t * enumeration_list, Agnode_t * enumeration,
		  char *alias)
{

  enumerationn_t *en = list_add (enumeration_list, enumerationn_t);

  char *str;
  if (alias == NULL)
    str = get_namedattr (enumeration, "name");
  else
    str = strdup (alias);

  linkedlist_t *list = list_new ();

  en->enumeration.name = str;
  en->enumeration.enumerators = list;

  Agedge_t *enumerator;
  for_each_child (enumerator, enumeration, cEnumerator)
  {
    char *str = get_namedattr (enumerator->head, "name");

    enumeratorn_t *node;
    int found = 0;
    for_each_list_element (list, node)
      {
      if (!strcmp (node->enumerator.name, str))
	{
	  found = 1;
	  break;
	}
    }

    if (found)
      {
	free (str);
	continue;
      }

    node = list_add (list, enumeratorn_t);
    node->enumerator.name = str;

    Agnode_t *value = resolve_path (enumerator->head, 1, "cLiteral");

    if (value)
      {
	char *str = get_namedattr (value, "value");
	node->enumerator.value = str;
      }
  }
}

inline void
struct_load (linkedlist_t * structn_list, Agnode_t * structn,
		  char *alias)
{

  structn_t *newstruct = list_add (structn_list, structn_t);

  assert(newstruct);
  assert(alias != NULL);
  newstruct->structdef.name = get_namedattr(structn, "name");
  newstruct->structdef.alias = strdup(alias);
  newstruct->structdef.fields = list_new ();

  Agedge_t *field;
  for_each_child (field, structn, cObject)
  {
    char * str = get_namedattr(field->head, "name");
    int found = 0;
    varn_t *node;
    for_each_list_element (newstruct->structdef.fields, node)
      {
	if (!strcmp (node->var->name, str))
	  {
	    found = 1;
	    break;
	  }
      }

    if(!found){
      node = list_add (newstruct->structdef.fields, varn_t);
      node->var = var_new(field->head);
      node->var->type->structdef = &newstruct->structdef;
      //printf("adding %s@%s\n ", var_str(node->var), alias);
    }

    free(str);
  }
}

static inline void
trunit_load (trunit_t * unit)
{

  Agnode_t *root = ROOT (unit->ast);
  Agedge_t *first = first_child (ROOT (unit->ast));
  Agedge_t *e1;

  for_each_child (e1, root, cCompilationScope)
  {
    Agnode_t *node = e1->head;

    // for each class declaration
    Agedge_t *e2;
    for_each_child (e2, node, cClassType)
    {
      Agnode_t *class = e2->head;
      // adds the module top level to the modules list
      modulen_t *node = list_add (unit->modules, modulen_t);
      node->module = module_new (class);
    }

    //loads enumeration and typedefs declarations

    for_each_child (e2, node, cEnumerationType)
    {
      //trunit_load_enumeration( unit, e2->head, NULL );
      enumeration_load (unit->enumerations, e2->head, NULL);
    }

    for_each_child (e2, node, cObjectList)
    {
      Agnode_t *enumeration =
	resolve_path (e2->head, 2, "cAliasType", "cEnumerationType");

      if (enumeration)
	{
	  Agnode_t *alias = resolve_path (e2->head, 1, "cAliasType");
	  char *str = get_namedattr (alias, "name");
	  //trunit_load_enumeration( unit, enumeration, str );
	  enumeration_load (unit->enumerations, enumeration, str);
	  free (str);
	}

      Agnode_t *structn = resolve_path(e2->head, 2, "cAliasType", "cClassType");
      if (structn){
	Agnode_t *alias = resolve_path (e2->head, 1, "cAliasType");
	char *str = get_namedattr (alias, "name");
	struct_load(unit->structs, structn, str);
	free(str);
      }
    }

    Agedge_t *alias;
    for_each_child (alias, node, cAliasType)
    {
      char *name = get_namedattr (alias->head, "name");
      Agnode_t *dst = resolve_path (alias->head, 1, "cBuiltInType");
      if (dst)
	{
	  char *value = get_namedattr (dst, "name");
	  aliasn_t *node = list_add (unit->aliases, aliasn_t);
	  node->alias.name = strdup (name);
	  node->alias.value = strdup (value);

	  free (value);
	}

      free (name);
    }

    Agedge_t *function;
    for_each_child (function, node, cFunction)
    {
      char *function_str = get_namedattr (function->head, "name");
      //printf( "Found a function %s\n", function_str );
      //free( str );
      Agnode_t *module = resolve_path (function->head, 1, "cNameReference");
      if (module)
	{
	  char *module_str = get_namedattr (module, "name");
	  //printf( "\tfrom %s\n", module_str );

	  modulen_t *node;
	  for_each_list_element (unit->modules, node)
	  {
	    if (!strcmp (node->module->name, module_str))
	      {
		methodn_t *method;
		for_each_list_element (node->module->methods, method)
		{
		  if (!strcmp (method->method->name, function_str))
		    {
		      //printf( "found %s's implementation\n", method->method->name );
		      assert (function->head);
		      method->method->implementation = function->head;
		      method->method->block =
			resolve_path (function->head, 1, "cBlock");
		      //print_ast( method->method->block );
		      assert (method->method->block);
		      break;
		    }
		}
		break;
	      }
	  }

	  free (function_str);
	  free (module_str);

	}
    }
  }

  modulen_t *modulen;
  for_each_list_element (unit->modules, modulen)
  {
    methodn_t *methodn;
    for_each_list_element (modulen->module->methods, methodn)
    {
      if (methodn->method->block)
	method_elaborate (methodn->method);
    }
  }


  return;
}

inline trunit_t *
trunit_new (char *filename)
{

  aginit ();

  FILE *fd = fopen (filename, "r");

  //assert (fd);
  if (!fd)
    {
      panic ("can't open %s ( parsing failed )", filename);
    }

  Agraph_t *g = agread (fd);

  fclose (fd);

  trunit_t *unit = malloc (sizeof (trunit_t));

  gunit = unit;
  assert(gunit);
  unit->ast = g;

  unit->modules = list_new ();
  unit->enumerations = list_new ();
  unit->structs = list_new ();
  unit->aliases = list_new ();

  trunit_load (unit);

  return unit;

}

inline void
trunit_free (trunit_t * unit)
{
  agclose (unit->ast);

  free (unit);
}
