#ifndef ASYNCROM_H
#define ASYNCROM_H
#include <assert.h>

#define ROM_TYPE(elem_type) static const elem_type



#define DEFINE_ASYNCROM(size, elem_type, index_type, rom_name ) \
        SC_MODULE(rom_name){\
          sc_in<bool>            clk;\
          sc_in<index_type >     addr;\
          sc_out<elem_type >     out_data;\
          void async_proc();\
          SC_CTOR(rom_name){\
            SC_METHOD(async_proc);\
            sensitive << addr;\
          }\
	};

#ifdef SIMULATION
#define SIM_ASSERT(size)   assert(addr.read() < size)
#else
#define SIM_ASSERT(size)
#endif



/*
 Below I do some obscure-voodoo-code with the ... (three-points) argument
 
 Here comes a explanation found in google:
 
  A macro can be declared to accept a variable number of arguments much as 
  a function can. The syntax for defining the macro is similar to that of 
  a function. Here is an example:

     #define eprintf(...) fprintf (stderr, __VA_ARGS__)

  This kind of macro is called variadic. When the macro is invoked, all the 
  tokens in its argument list after the last named argument (this macro has 
  none), including any commas, become the variable argument. This sequence 
  of tokens replaces the identifier __VA_ARGS__ in the macro body wherever 
  it appears. Thus, we have this expansion:

  eprintf ("%s:%d: ", input_file, lineno)
     ==>  fprintf (stderr, "%s:%d: ", input_file, lineno)

 
     

 So, I use this __VA_ARGS__ to get the values passed to the ROM buffer const array;
 
 The use of the macro IMPL_SYNCROM is like these:
 IMPL_SYNCROM( size, elem_type, rom_name, 123, 456, 789, 111, 52, 623 );
  
 Where the numbers { 123, 456, 789, 111, 52, 623 } are the values of the array in the ROM buffer.
*/

#define IMPL_ASYNCROM(size, elem_type, rom_name, ...) \
void rom_name::async_proc(){\
  ROM_TYPE(elem_type)    buff[size] = {  __VA_ARGS__  } ;\
  SIM_ASSERT(size); \
  out_data = buff[addr.read()];\
}\

#endif
