#ifndef WINDOW_H
#define WINDOW_H
#include <systemc.h>
#include "multiplier.h"
#include "ocp.h"
#include "window-vll.h"
#include "window-syncrom.h"


SC_MODULE(window){

    sc_in<bool>          clk;
    sc_in<bool>          Reset_n;
    sc_in<bool>          in_trans;
    sc_in<sc_uint<32> >  data_in;
    sc_in<sc_uint<2> >   in_nch;
//    sc_out<sc_uint<2> >  out_nch;
    sc_out<bool>         in_ready;

    sc_out<sc_uint<1> >  MAddr;
    sc_out<sc_uint<3> >  MCmd;
    sc_out<sc_uint<32> > MData;
    sc_in<bool>          SCmdAccept;
    sc_in<sc_uint<8> >   SData;
    sc_in<sc_uint<2> >   SResp;

    sc_signal<bool> estado_pre_read;
    sc_signal<bool> estado_read;
    /*sc_out<sc_uint<32> > mula_out;
    sc_out<sc_uint<32> > mulb_out;
    sc_out<sc_uint<32> > mulc_out;

    sc_out<sc_uint<10> > mem_addr;
    sc_out<sc_uint<32> > mem_in;

    sc_out<sc_uint<32> > mem_out;
    sc_out<bool> mem_we;*/


  void fsm();
  void comb_sigs();

    sc_signal<sc_uint<32> > mul_a;
    sc_signal<sc_uint<18> > mul_b;
    sc_signal<sc_uint<32> > mul_c;

    multiplier_16_18 *mul0;



    vllrom *vllrom0; // creates a 512x18 ROM

    // signals bound to the rom-memory pins
    sc_signal<sc_uint<9> > vllrom_addr;
    sc_signal<sc_uint<18> > vllrom_out;




    vllram *vll0; // creates a 1024x32 RAM

    // signals bound to the memory pins
    sc_signal<sc_uint<10> > vll_addr;
    sc_signal<sc_uint<32> > vll_in;
    sc_signal<sc_uint<32> > vll_out;
    sc_signal<bool> vll_we;

    
    


    enum state_t {pre_read_st, read_st, proc_st, mem_read_proc_st, mem_read_send_st, send_st
      ,mem_read_send2_st,
      mem_read_proc2_st
    
    };
    sc_signal<state_t> state;
    //sc_signal<sc_uint<32> > vll[2][512];
    sc_signal<sc_uint<4> >  i;
    sc_signal<sc_uint<5> >  j;
    sc_signal<sc_uint<9> >  ind;
    sc_signal<sc_uint<32> > Sj;
    //sc_signal<sc_uint<4> >  block[2];
    //sc_signal<sc_uint<5> >  valid[2];

    sc_signal<sc_uint<4> >  block_0; // to avoid buffers
    sc_signal<sc_uint<4> >  block_1;

    sc_signal<sc_uint<5> >  valid_0; // to avoid buffers
    sc_signal<sc_uint<5> >  valid_1;
    
    sc_signal<sc_uint<1> >  ch;
    sc_signal<sc_uint<4> >  k;
    sc_signal<sc_uint<9> >  l;
    sc_signal<sc_uint<2> >  nch;
    sc_signal<sc_uint<2> >  prev_nch;

//    void dump();
    
  SC_CTOR(window){
     /*SC_METHOD(dump);
     sensitive << mul_a << mul_b << mul_c << vll_in << vll_addr << vll_out << vll_we;*/
     
    vll0 = new vllram("vll0");
    vll0->clk(clk);
    vll0->in_data(vll_in);
    vll0->out_data(vll_out);
    vll0->we(vll_we);
    vll0->addr(vll_addr);
    
    
    vllrom0 = new vllrom("vllrom0");
    vllrom0->clk(clk);
    vllrom0->out_data( vllrom_out );
    vllrom0->addr( vllrom_addr );
    
    

    mul0 = new multiplier_16_18("mul0");
        mul0->a(mul_a);
        mul0->b(mul_b);
        mul0->c(mul_c);
        SC_METHOD(fsm);
        sensitive_pos << clk;
        SC_METHOD(comb_sigs);
        sensitive << i;
        sensitive << k;
        sensitive << j;
        sensitive << ind;
        sensitive << ch;
        sensitive << Sj;
        sensitive << l;
        sensitive << state;
        sensitive << mul_c;
        sensitive << nch;
        sensitive << in_nch;
        sensitive << in_ready;
        sensitive << in_trans;
        sensitive << prev_nch;
	
	/// below: process sensitive to the memory pins
	sensitive << vll_we;
	sensitive << vll_addr;
	// We dont need the sensitive of vll_in and vll_out, becouse now these values is modified in the state machine
	//sensitive << vll_in; 
	//sensitive << vll_out;
  }
};
#endif
